<?php

/**
 * Contains the base objects of the Smart Modules
 *
 * @license GNU
 * @author marcan <marcan@smartfactory.ca> 
 * @version $Id: smartobject.php,v 1.1.2.1 2005/07/24 17:14:40 malanciault Exp $
 * @link http://www.smartfactory.ca The SmartFactory
 * @package SmartX
 * @subpackage SmartModules
 */

/** 
 * In order to run properly, the SmartObject need the SmartLanguage module to be installed
 * So the first thing we will be checking, is if the module is properly installed
 */
$modules_handler = xoops_gethandler('module');
$smartlanguage_module = $modules_handler->getByDirname('smartlanguage');

static $xoops_language_tags;

if(!isset($xoops_language_tags)) {

	if (!$smartlanguage_tag_handler) {
		$smartlanguage_tag_handler = xoops_getmodulehandler('tag', 'smartlanguage')	;
	}
	$xoops_language_tags =& $smartlanguage_tag_handler->getObjects();
}

if (!$smartlanguage_module || ($smartlanguage_module->getVar('isactive') == false) || !$xoops_language_tags) {
	?>
	<div style="margin-top: 20px;  margin-left: 200px; margin-right: 200px; border: 1px grey solid; padding-right: 10px; padding-bottom: 10px; padding-left: 10px;">
	<h2>SmartLanguage module is not installed and configured correctly</h2>
	This site is using a SmartModule based in the <b>SmartObject</b> classes. In order to run properly, SmartLanguage module needs to be correctly installed and configured.<br />
	<br />
	<b>SmartObject</b> classes are only at a beta stage and this will be more properly handled in the futur. Please forgive us for any inconvenience.<br />
	<br />
	Please install the SmartLanguage module to solve this problem. For more informations, don't hesitate to contact  us at <a href="mailto:info@smartfactory.ca">info@smartfactory.ca</a>.<br />
	<br />
	The SmartFactory Team<br />
	<a href="http://www.smartfactory.ca">www.smartfactory.ca</a>
	</div>
	<?php
exit;	
}

/** Constant to specify that we want all objects with or without languages */
define("_SMART_ALL_OBJECTS", 'all_objects');
/** Constant to specify that we want all objects that have languages, whatever the language */
define("_SMART_ANY_LANGUAGES", 'any_languages');

/** Constant to specify that the main object id will be used as array key in getObjects */
define("_SMART_KEY_TYPE_MAIN_ID", 'main_id');
/** Constant to specify that the parent object id will be used as array key in getObjects */
define("_SMART_KEY_TYPE_PARENT_ID", 'parent_id');
/** Constant to specify that non key will be used as array key in getObjects */
define("_SMART_KEY_TYPE_NONE", 'none');


/**
 * Smart structure class
 *
 * Object use to create the structure of a SmartObject
 *
 * @package SmartX
 * @author marcan <marcan@smartfactory.ca> 
 * @link http://www.smartfactory.ca The SmartFactory 
 */ 
class SmartStructure
{
	/**
     * Complete structure of the SmartObject
     * 
     * @var	array
     */	
	var $_structure;
	
	
   /**
    * Constructor
    */	
	function SmartStructure() 
	{
		$this->_structure = array();
	}	
	
	function addField($key, $data_type, $value = null, $required = false, $maxlength = null, $multilingual, $options = '')
	{
		$newField = array();
		$newField['key'] = $key;
		$newField['data_type'] = $data_type;
		$newField['value'] = $value;
		$newField['required'] = $required;
		$newField['maxlength'] = $maxlength;
		$newField['multilingual'] = $multilingual;
		$newField['options'] = $options;		
		$this->_structure[] = $newField;
		unset ($newField);
	}

	function getStructure() 
	{
		return $this->_structure;
	}		
}


/**
 * Smart base object class
 *
 * Base class of a single object
 *
 * @package SmartX
 * @author marcan <marcan@smartfactory.ca> 
 * @link http://www.smartfactory.ca The SmartFactory 
 */ 

class SmartObject extends XoopsObject
{
    /**
    * initialize variables for the object
    * 
    * @access public
    * @param string $key
    * @param int $data_type  set to one of XOBJ_DTYPE_XXX constants (set to XOBJ_DTYPE_OTHER if no data type ckecking nor text sanitizing is required)
    * @param mixed
    * @param bool $required  require html form input?
    * @param int $maxlength  for XOBJ_DTYPE_TXTBOX type only
    * @param bool $multilingual is this variable multilingual ?
    * @param string $option  does this data have any select options?
    */
    function initVar($key, $data_type, $value = null, $required = false, $maxlength = null, $multilingual, $options = '')
    {
        $this->vars[$key] = array('value' => $value, 'required' => $required, 'data_type' => $data_type, 'maxlength' => $maxlength, 'changed' => false, 'multilingual' => $multilingual, 'options' => $options);
    }
}

/**
 * Smart base handler object class
 *
 * This class is an abstract class of handler classes that are responsible for providing
 * data access mechanisms to the data source of its corresponsing data objects
 *
 * @package SmartX
 * @author marcan <marcan@smartfactory.ca> 
 * @link http://www.smartfactory.ca The SmartFactory 
 */ 
class SmartObjectHandler extends XoopsObjectHandler 
{
	/**
     * Name of child class
     * 
     * @var	string
     */
	var $classname = '';
	
    /**
     * Related table name
     * 
     * @var string
     */
     var $_dbtable = '';
     
    /**
     * key field name
     * 
     * @var string
     */
     var $_key_field = '';    

    /**
     * caption field name
     * 
     * @var string
     */
     var $_caption_field = '';  
     
	/**
     * Name of parent class
     * 
     * @var	string
     */
	var $_parent_classname = '';
	
    /**
     * Related parent table name
     * 
     * @var string
     */
     var $_parent_dbtable = '';
     
    /**
     * parent key field name
     * 
     * @var string
     */
     var $_parent_key_field = '';    

    /**
     * parent caption field name
     * 
     * @var string
     */
     var $_parent_caption_field = '';       

    /**
     * multilingual table
     * 
     * @var string
     */
     var $_multilingual_table = '';  
     
    /**
     * select fields
     * 
     * @var string
     */
     var $_select_fields = false;  

    /**
     * multilingual select fields
     * 
     * @var string
     */
     var $_multilingual_select_fields = false;  

    /**
	 * Create the list of fields to be included in the SELECT query
	 * 
	 * @param object $criteria {@link CriteriaElement} to match
	 * @param string $languageid language to match : can be any existing language or _SMART_ALL_OBJECTS or _SMART_ANY_LANGUAGES
	 * @return	string fields to be included
	 */	
    function createSelectFields()
    {
    	if (!isset($main_fields)) {
    		global $obj_structure;
			$fields = $obj_structure->getStructure();
			$main_fields = array();
			$text_fields = array();
			foreach ($fields as $field) {
				if ($field['multilingual']) {
					$text_fields[] = 'texttable.' . $field['key'];
				} else {
					$main_fields[] = 'maintable.' . $field['key'];
				}
			}
			$this->_select_fields = implode(', ', $main_fields);
			$this->_multilingual_select_fields = implode(', ', $text_fields);
			unset($main_fields);
			unset($text_fields);
    	}
    }    
     
     /**
	 * Create a "SELECT" SQL query
	 * 
	 * @param object $criteria {@link CriteriaElement} to match
	 * @param string $languageid language to match : can be any existing language or _SMART_ALL_OBJECTS or _SMART_ANY_LANGUAGES
	 * @param bool $countQuery true if this is a SELECT COUNT query, false if it's a normal SELECT
	 * @param mixed $notNullFields additionnal WHERE clause to filter records without some empty fields
	 * @return	string SQL query
	 */	
    function _selectQuery($criteria = null, $languageid=_SMART_ALL_OBJECTS, $countQuery=false, $notNullFields='')
    {
    	$this->createSelectFields();
    	if ($countQuery) {
    		$selectFields = 'COUNT(maintable.' . $this->_key_field . ')';
    	} else {
    		$selectFields = $this->_select_fields . ', ' . $this->_multilingual_select_fields;
    	}
    	
    	switch ($languageid) {
    		case _SMART_ALL_OBJECTS :
				$sql = 	sprintf('SELECT ' . $selectFields . ' FROM %s AS maintable LEFT JOIN %s AS texttable ON maintable.' . $this->_key_field . ' = texttable.' . $this->_key_field . ' ', 
						$this->db->prefix($this->_dbtable),
						$this->db->prefix($this->_multilingual_table)
						);
    		break;
    		
    		case _SMART_ANY_LANGUAGES :
				$sql = 	sprintf('SELECT ' . $selectFields . ' FROM %s AS maintable INNER JOIN %s AS texttable ON maintable.' . $this->_key_field . ' = texttable.' . $this->_key_field . ' ', 
						$this->db->prefix($this->_dbtable),
						$this->db->prefix($this->_multilingual_table)
						);
    		break;
    		
    		// If $languageid is != _SMART_ALL_LANGUAGES or _SMART_ANY_LANGUAGES, a real language  was inputed
    		default:
				// Check to see if the language is present on the site
				global $xoops_language_tags;
				
				if (array_key_exists($languageid, $xoops_language_tags)) {
    				$sql = 	sprintf('SELECT ' . $selectFields . ' FROM %s AS maintable INNER JOIN %s AS texttable ON maintable.' . $this->_key_field . ' = texttable.' . $this->_key_field . ' ', 
							$this->db->prefix($this->_dbtable),
							$this->db->prefix($this->_multilingual_table)
							);
					// Add the WHERE clause to select only this language
					if (!(isset($criteria) && is_subclass_of($criteria, 'criteriaelement'))) {
						// if the $criteria does not exists, let's create one	
						$criteria = new CriteriaCompo();
					}
					$criteria->add(new Criteria('texttable.languageid', $languageid));
				} else {
					// The $languageid is not a language present on the site.
					// Todo : how do we handle this...
				}
    		break;
    	}
    	
        if(isset($criteria) && is_subclass_of($criteria, 'criteriaelement')) {
        	$whereClause = $criteria->renderWhere();
        	If ($whereClause != 'WHERE ()') {
				$sql .= ' ' . $whereClause;
				If (!empty($notNullFields)) {
					$sql .= $this->notNullFieldClause($notNullFields, true);
				}        		
        	} elseif (!empty($notNullFields)) {
				$sql .= " WHERE " . $this->notNullFieldClause($notNullFields);
			}
        	
            if($criteria->getSort() != '') {
                $sql .= ' ORDER BY ' . $criteria->getSort() . ' 
                    ' .$criteria->getOrder();
            }
            
		} elseif (!empty($notNullFields)) {
			$sql .= $sql .= " WHERE " . $this->notNullFieldClause($notNullFields);
		}
        
        return $sql;
    }
    
    /**
	 * retrieve number of objects in the database
	 * 
	 * @param object $criteria {@link CriteriaElement} conditions to be met
	 * @param string $languageid language to match : can be any existing language or _SMART_ALL_OBJECTS or _SMART_ANY_LANGUAGES
	 * @param bool $id_as_key Should the key field be used as array key
	 * @param mixed $notNullFields additionnal WHERE clause to filter records without some empty fields
	 *
	 * @return integer objects count
	 * @access	public	
	 */	

    function getCount2($criteria = null, $languageid = _SMART_ALL_OBJECTS, $key_type = _SMART_KEY_TYPE_MAIN_ID, $notNullFields='')
    {
        $sql = $this->_selectQuery($criteria, $languageid, true, $notNullFields);
        
        if (isset($criteria)) {
            $limit = $criteria->getLimit();
            $start = $criteria->getStart();
        }
        
       // echo "<br />$sql<br />";
        
        $result = $this->db->query($sql, $limit, $start);
        
		if (!$result) {
			return 0;
		}
		list($count) = $this->db->fetchRow($result);  
		return $count;      
    }
    
    /**
	 * retrieve objects from the database
	 * 
	 * @param object $criteria {@link CriteriaElement} conditions to be met
	 * @param string $languageid language to match : can be any existing language or _SMART_ALL_OBJECTS or _SMART_ANY_LANGUAGES
	 * @param bool $id_as_key Should the key field be used as array key
	 * @param mixed $notNullFields additionnal WHERE clause to filter records without some empty fields
	 *
	 * @return array array of SmartObject objects
	 * @access	public	
	 */	
    function &getObjects2($criteria = null, $languageid = _SMART_ALL_OBJECTS, $key_type = _SMART_KEY_TYPE_MAIN_ID, $notNullFields='')
    {
    	$ret    = array();
        $limit  = $start = 0;
        $sql    = $this->_selectQuery($criteria, $languageid, $notNullFields);
        
        if (isset($criteria)) {
            $limit = $criteria->getLimit();
            $start = $criteria->getStart();
        }
        
        //echo "<br />$sql<br />";
        
        $result = $this->db->query($sql, $limit, $start);
        // If no records from db, return empty array
        if (!$result) {
            return $ret;
        }
        
        // Add each returned record to the result array
        while ($myrow = $this->db->fetchArray($result)) {
            $obj = new $this->classname($myrow);
            
            switch ($key_type) {
            	case _SMART_KEY_TYPE_PARENT_ID :
            	$ret[$myrow[$this->_parent_key_field]] =& $obj;
            	break;
            	
            	case _SMART_KEY_TYPE_MAIN_ID :
            	$ret[$myrow[$this->_key_field]] =& $obj;
            	break;
            	
            	case _SMART_KEY_TYPE_NONE :
            	default:
            	$ret[] =& $obj;
            	break;
            }
            if (!$id_as_key) {
                
            } else {
            	
            }
            unset($obj);
        }
        return $ret;
    }  
    
	function notNullFieldClause($notNullFields='', $withAnd=false)
	{
		$ret = '';
		If ($withAnd) {
			$ret .= " AND ";
		}
		If ( !empty($notNullFields) && (is_array($notNullFields)) ) {
			foreach ($notNullFields as $v) {
				$ret .= " ($v IS NOT NULL AND $v <> ' ' )";
			}
		} elseif ( !empty($notNullFields)) {
			$ret .= " ($notNullFields IS NOT NULL AND $notNullFields <> ' ' )";
		}
		return $ret;
	}      
          
}
?>